@@ -1,34 +1,8 @@
-# -*- coding: utf-8 -*-
-#
-# Copyright (c) 2011-2018, Cédric Krier
-# Copyright (c) 2011-2018, B2CK
-# All rights reserved.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-#     * Redistributions of source code must retain the above copyright
-#       notice, this list of conditions and the following disclaimer.
-#     * Redistributions in binary form must reproduce the above copyright
-#       notice, this list of conditions and the following disclaimer in the
-#       documentation and/or other materials provided with the distribution.
-#     * Neither the name of the <organization> nor the
-#       names of its contributors may be used to endorse or promote products
-#       derived from this software without specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
-# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
-# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
-# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
-# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
+# This file is part of python-sql.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from itertools import chain
 
-from sql import Expression, Flavor, FromItem, Window
+from sql import Expression, Select, CombiningQuery, Flavor, FromItem, Window
 
 __all__ = ['Abs', 'Cbrt', 'Ceil', 'Degrees', 'Div', 'Exp', 'Floor', 'Ln',
     'Log', 'Mod', 'Pi', 'Power', 'Radians', 'Random', 'Round', 'SetSeed',
@@ -72,6 +46,8 @@ class Function(Expression, FromItem):
     def _format(value):
         if isinstance(value, Expression):
             return str(value)
+        elif isinstance(value, (Select, CombiningQuery)):
+            return '(%s)' % value
         else:
             return Flavor().get().param
 
@@ -89,7 +65,7 @@ class Function(Expression, FromItem):
             return Mapping(*self.args).params
         p = []
         for arg in self.args:
-            if isinstance(arg, Expression):
+            if isinstance(arg, (Expression, Select, CombiningQuery)):
                 p.extend(arg.params)
             else:
                 p.append(arg)
@@ -106,9 +82,9 @@ class FunctionKeyword(Function):
         if Mapping:
             return str(Mapping(*self.args))
         return (self._function + '('
-            + ' '.join(chain(*list(zip(
+            + ' '.join(chain(*zip(
                         self._keywords,
-                        list(map(self._format, self.args))))))[1:]
+                        map(self._format, self.args))))[1:]
             + ')')
 
 
@@ -474,6 +450,8 @@ class AtTimeZone(Function):
             return str(Mapping(self.field, self.zone))
         if isinstance(self.zone, Expression):
             zone = str(self.zone)
+        elif isinstance(self.zone, (Select, CombiningQuery)):
+            zone = '(%s)' % self.zone
         else:
             zone = flavor.param
         return '%s AT TIME ZONE %s' % (str(self.field), zone)
@@ -483,7 +461,7 @@ class AtTimeZone(Function):
         Mapping = Flavor.get().function_mapping.get(self.__class__)
         if Mapping:
             return Mapping(self.field, self.zone).params
-        if isinstance(self.zone, Expression):
+        if isinstance(self.zone, (Expression, Select, CombiningQuery)):
             return self.field.params + self.zone.params
         else:
             return self.field.params + (self.zone,)
@@ -523,7 +501,10 @@ class WindowFunction(Function):
         filter_ = ''
         if self.filter_:
             filter_ = ' FILTER (WHERE %s)' % self.filter_
-        over = ' OVER "%s"' % self.window.alias
+        if self.window.has_alias:
+            over = ' OVER "%s"' % self.window.alias
+        else:
+            over = ' OVER (%s)' % self.window
         return function + filter_ + over
 
     @property
@@ -531,6 +512,8 @@ class WindowFunction(Function):
         p = list(super(WindowFunction, self).params)
         if self.filter_:
             p.extend(self.filter_.params)
+        if not self.window.has_alias:
+            p.extend(self.window.params)
         return tuple(p)
 
 
